@@ -372,4 +372,12 @@
     |#
     #t))
 
+(when #f
+  ;; bug 65849
+  (let* ((code (string-append
+                "#warning \"warn me!\"\n" 
+                "#warning \"oops\"\n"))
+         (tree (parse-string code)))
+    #t))
+
 ;; --- last line ---
