@@ -5,7 +5,7 @@
 (use-modules (yaml))
 (use-modules (ice-9 pretty-print))
 
-(let ((yaml (read-yaml-file "demo1.yml")))
+(let ((yaml (read-yaml-file "yaml-01.yml")))
   (pretty-print yaml))
 
 ;; --- last line ---
