@@ -1,6 +1,6 @@
 ;; nyacc/lang/nx-util.scm - run-time library
 
-;; Copyright (C) 2018 Matthew R. Wette
+;; Copyright (C) 2018,2024 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -56,13 +56,18 @@
 
 (define-module (nyacc lang nx-lib)
   #:export (nx-get-method
+            nx-use-module
+            nx-C-predicate
+            nx-undefined
             ;;
             make-nx-hash-table nx-hash-ref nx-hash-set!
             nx-hash-add-lang nx-hash-lang-ref nx-hash-lang-set! %nx-lang-key
+            mat-disp vec-disp
             ;;
             install-inline-language-evaluator
             uninstall-inline-language-evaluator)
-  )
+  #:use-module (nyacc lang nx-printf))
+
 (define (sferr fmt . args) (apply simple-format (current-error-port) fmt args))
 (use-modules (ice-9 pretty-print))
 (define (pperr exp)
@@ -71,8 +76,8 @@
 (define nx-undefined (if #f #f))
 
 ;; @deffn {Procedure} nx-get-method obj name
-;; find a 
-;; @end deffn 
+;; find a
+;; @end deffn
 (define (nx-get-method obj name)
   #f)
 
@@ -113,34 +118,83 @@
 (define (nx-hash-set! htab key val)
   (hashq-set! htab key val))
 
-(define (sprintf fmt . args)
-  (define rls reverse-list->string)
-  (define (numstr->string val)
-    (if (string? val) val (number->string val)))
-  (define (escape ch)
-    (case ch
-      ((#\\) #\\)
-      ((#\n) #\newline)
-      (else ch)))
-  (with-input-from-string fmt
-    (lambda ()
-      (let loop ((stl '()) (chl '()) (ch (read-char)) (args args))
-        ;;(sf "ch=~S\n" ch)
-        (cond
-         ((eof-object? ch)
-          (apply string-append (reverse (cons (rls chl) stl))))
-         ((char=? ch #\%)
-          (let ((ch1 (read-char)))
-            (case ch1
-              ((#\\) (loop stl (cons (escape ch1) chl) (read-char) args))
-              ((#\%) (loop stl (cons ch1 chl) (read-char) args))
-              ((#\s) (loop (cons* (car args) (rls chl) stl) '()
-                           (read-char) (cdr args)))
-              ((#\d #\f) (loop (cons* (numstr->string (car args)) (rls chl) stl)
-                               '() (read-char) (cdr args)))
-              (else (error "sprintf: unknown % char")))))
-         (else
-          (loop stl (cons ch chl) (read-char) args)))))))
+;; ====================================
+;; code to be organized
+
+;; routine to make a single arg predicate into a zero? predcate
+
+;; ???
+(define (nx-C-predicate proc)
+  (lambda (arg)
+    (let ((res (proc arg)))
+      (if res 1 0))))
+
+(use-modules (ice-9 match))
+(define (add-src-prop-attr sx)
+  (define (p2l pair) (list (car pair) (cdr pair)))
+  (define (ins-src-prop sx attr)
+    (cons (map p2l (source-properties sx)) attr))
+  (match sx
+    (`(,tag (@ . ,attr) . ,rest)
+     `(tag (@ . ,(ins-src-prop sx attr)) . ,(map add-src-prop-attr rest)))
+    (`(,tag . ,rest)
+     (let ((src-prop (source-properties sx)))
+       (if (pair? src-prop)
+	   `(,tag (@ . ,(map p2l src-prop)) . ,(map add-src-prop-attr rest))
+	   `(,tag . ,(map add-src-prop-attr rest)))))
+    (_ sx)))
+(export add-src-prop-attr)
+
+(define (nx-error fmt . args)
+  (throw 'error (apply simple-format #f fmt args)))
+
+(define (nx-use-module path)
+  (module-use! (current-module) (resolve-interface path)))
+
+;; === matrices and vectors ========
+
+(define (mat-disp/strict array port format)
+  (let* ((conv (array-type array))
+         (dims (array-dimensions array))
+         (dimz (map (lambda (i) (min i 8)) dims))
+         (spec (parse-format-string format)))
+    (do ((i 0 (1+ i))) ((= i (list-ref dimz 0)))
+      (do ((j 0 (1+ j))) ((= j (list-ref dimz 1)))
+        (display " " port)
+        (apply-fmt port spec (array-ref array i j)))
+      (newline port))))
+
+(define* (mat-disp array #:optional (port #t) (format "%12.5e"))
+  (cond
+   ((eq? #f port)
+    (let ((strp (open-output-string)))
+      (mat-disp/strict array strp format)
+      (get-output-string strp)))
+   ((eq? #t port)
+    (mat-disp/strict array (current-output-port) format))
+   (else
+    (mat-disp/strict array port format))))
+
+(define (vec-disp/strict array port format)
+  (let* ((conv (array-type array))
+         (dims (array-dimensions array))
+         (dimz (map (lambda (i) (min i 8)) dims))
+         (spec (parse-format-string format)))
+    (do ((i 0 (1+ i))) ((= i (list-ref dimz 0)))
+      (display " " port)
+      (apply-fmt port spec (array-ref array i))
+      (newline port))))
+
+(define* (vec-disp array #:optional (port #t) (format "%12.5e"))
+  (cond
+   ((eq? #f port)
+    (let ((strp (open-output-string)))
+      (vec-disp/strict array strp format)
+      (get-output-string strp)))
+   ((eq? #t port)
+    (vec-disp/strict array (current-output-port) format))
+   (else
+    (vec-disp/strict array port format))))
 
 ;;; === in-line reading =========================================================
 
@@ -227,7 +281,7 @@
   (if #f #f))
 
 ;; @deffn {Procedure} uninstall-inline-language-evaluator
-;; Clear the reader macro @code{#<}.  
+;; Clear the reader macro @code{#<}.
 ;; @end deffn
 (define (uninstall-inline-language-evaluator)
   "- Procedure: uninstall-inline-language-evaluator
