@@ -29,12 +29,15 @@
 
 (define-module (nyacc lang sx-util)
   #:export (make-sx
-	    sx-tag sx-attr sx-tail sx-length sx-ref sx-ref*
-	    sx-has-attr? sx-attr-ref sx-attr-add sx-attr-add* sx-attr-set!
-	    sx-find
-	    sx-split sx-split* sx-join sx-join* sx-cons* sx-list
-	    sx-unitize
-	    sx-match sx-match-tail)
+            sx-tag sx-attr sx-tail sx-length sx-ref sx-ref*
+            sx-has-attr? sx-attr-ref sx-attr-add sx-attr-add* sx-attr-set!
+            sx-find
+            sx-split sx-split* sx-join sx-join* sx-cons sx-cons* sx-list
+            sx-unitize
+            sx-match sx-match-tail
+            ;; deprecated:
+            sx-join sx-join*
+            )
   #:use-module ((srfi srfi-1) #:select (find fold fold-right append-reverse)))
 (cond-expand
   (mes)
@@ -67,17 +70,17 @@
 ;; @end deffn
 (define (make-sx tag attr . elts)
   (let ((tail (fold-right
-	       (lambda (elt sx)
-		 (cond
-		  ((pair? elt) (cons elt sx))
-		  ((string? elt) (cons elt sx))
-		  (else sx)))
-	       '() elts)))
+               (lambda (elt sx)
+                 (cond
+                  ((pair? elt) (cons elt sx))
+                  ((string? elt) (cons elt sx))
+                  (else sx)))
+               '() elts)))
     (if (pair? attr)
-	(if (eq? '@ (car attr))
-	    (cons* tag attr tail)
-	    (cons* tag `(@ . ,attr) tail))
-	(cons tag tail))))
+        (if (eq? '@ (car attr))
+            (cons* tag attr tail)
+            (cons* tag `(@ . ,attr) tail))
+        (cons tag tail))))
 
 ;; @deffn {Procedure} sx-length sx => <int>
 ;; Return the length, don't include attributes, but do include tag
@@ -161,8 +164,14 @@
   (cond
    ((symbol? tag-or-path)
     (find (lambda (node)
-	    (and (pair? node) (eqv? tag-or-path (car node))))
-	  sx))
+            (and (pair? node) (eqv? tag-or-path (car node))))
+          sx))
+   ((pair? tag-or-path)
+    (let ((sym (car tag-or-path)))
+      (and=> (sx-find (car tag-or-path) sx)
+             (lambda (sub-sx) (sx-find (cdr tag-or-path) sub-sx)))))
+   ((null? tag-or-path)
+    sx)
    (else
     (error "sx-find: expecting first arg to be tag or sxpath"))))
 
@@ -179,39 +188,49 @@
 ;; @end deffn
 (define (sx-attr sx)
   (if (and (pair? (cdr sx))
-	   (pair? (cadr sx))
-	   (eqv? '@ (caadr sx)))
+           (pair? (cadr sx))
+           (eqv? '@ (caadr sx)))
       (cdadr sx)
       '()))
 
-;; @deffn {Procedure} sx-attr-ref sx|node|tail key => val
+;; @deffn {Procedure} sx-attr-ref sx|node|tail|#f key => val
 ;; Return an attribute value given the key, or @code{#f}.
-;; Also works if passed the attribute node @code{(@ ...)} or its tail.
+;; Also works if passed the attribute node @code{(@ ...)}, or its tail,
+;; or @code{#f}.
 ;; @end deffn
 (define (sx-attr-ref sx key)
-  (let ((attr-tail (cond ((null? sx) sx)
-			 ((pair? (car sx)) sx)
-			 ((eqv? '@ (car sx)) (car sx))
-			 ((sx-attr sx))
-			 (else '()))))
+  (let ((attr-tail (cond ((not sx) '())
+                         ((null? sx) sx)
+                         ((pair? (car sx)) sx)
+                         ((eqv? '@ (car sx)) (car sx))
+                         ((sx-attr sx))
+                         (else '()))))
     (and=> (assq-ref attr-tail key) car)))
 
+;; map to #f or '(@ ...)
+(define (encode-attr attr)
+  (cond
+   ((not attr) #f)
+   ((null? attr) #f)
+   ((eq? '@ (car attr)) (if (null? (cdr attr)) #f attr))
+   (else (cons '@ attr))))
+
 ;; @deffn {Procedure} sx-attr-add sx key-or-pair [val]
 ;; Add attribute to sx, passing either a key-val pair or key and val.
 ;; @end deffn
 (define* (sx-attr-add sx key-or-pair #:optional val)
   (let* ((pair (if val (list key-or-pair val) key-or-pair))
-	 (key (car pair)) (val (cadr pair)))
+         (key (car pair)) (val (cadr pair)))
     (cons
      (sx-tag sx)
      (if (sx-has-attr? sx)
-	 (cons `(@ pair
-		   ,@(let loop ((atl (sx-attr sx)))
-		       (cond ((null? atl) '())
-			     ((eq? key (caar atl)) (loop (cdr atl)))
-			     (else (cons (car atl) (loop (cdr atl)))))))
-	       (cddr sx))
-	 (cons `(@ ,pair) (cdr sx))))))
+         (cons `(@ pair
+                   ,@(let loop ((atl (sx-attr sx)))
+                       (cond ((null? atl) '())
+                             ((eq? key (caar atl)) (loop (cdr atl)))
+                             (else (cons (car atl) (loop (cdr atl)))))))
+               (cddr sx))
+         (cons `(@ ,pair) (cdr sx))))))
 
 ;; @deffn {Procedure} sx-attr-add* sx key val [key val [@dots{} ]] => sx
 ;; Add key-val pairs. @var{key} must be a symbol and @var{val} must be
@@ -219,11 +238,11 @@
 ;; @end deffn
 (define (sx-attr-add* sx . rest)
   (let* ((attrs (sx-attr sx))
-	 (attrs (let loop ((kvl rest))
-		  (if (null? kvl) attrs
-		      (cons (list (car kvl) (cadr kvl)) (loop (cddr kvl)))))))
+         (attrs (let loop ((kvl rest))
+                  (if (null? kvl) attrs
+                      (cons (list (car kvl) (cadr kvl)) (loop (cddr kvl)))))))
     (cons* (sx-tag sx) (cons '@ attrs)
-	   (if (sx-has-attr? sx) (cddr sx) (cdr sx)))))
+           (if (sx-has-attr? sx) (cddr sx) (cdr sx)))))
 
 ;; @deffn {Procedure} sx-attr-set! sx key val
 ;; Set attribute for sx.  If no attributes exist, if key does not exist,
@@ -232,49 +251,39 @@
 (define (sx-attr-set! sx key val)
   (if (sx-has-attr? sx)
       (let ((attr (cadr sx)))
-	(set-cdr! attr (assoc-set! (cdr attr) key (list val))))
+        (set-cdr! attr (assoc-set! (cdr attr) key (list val))))
       (set-cdr! sx (cons `(@ (,key ,val)) (cdr sx))))
   sx)
 
-;; @deffn {Procedure} sx-cons* tag attr exp ... tail => sx
-;; @deffnx {Procedure} sx-list tag attr exp ... => sx
-;; Generate the tag and the attr list if it exists.  Note that
-;; The following are equivalent:
-;; @example
-;; (sx-cons* tag attr elt1 elt2 '())
-;; (sx-list tag attr elt1 elt2)
-;; @end example
-;; @noindent
-;; Expressions that are @code{#f} or @code{'()} will be skipped;
-;; they should be strings or pairs.
+;; @deffn {Procedure} sx-cons tag attr tail => sexp
+;; @deffnx {Procedure} sx-cons* tag attr exp ... tail => sexp
+;; @deffnx {Procedure} sx-list tag attr exp ... => sexp
+;; Build an SXML element by its parts.  If @var{ATTR} is @code{#f},
+;; @code{'()} or @code{(@)} it will not be included. 
+;; @code{sx-cons*} and @code{sx-list} will remove any expressions @var{exp}
+;; that are @code{#f} or @code{'()}.
 ;; @end deffn
+(define (sx-cons tag attr tail)
+  (let ((attr (encode-attr attr)))
+    (if attr (cons* tag attr tail) (cons tag tail))))
 (define (sx-cons* tag attr . rest)
-  (if (null? rest) (error "sx-cons: expecing tail"))
-  (let ((attr (cond
-	       ((not attr) #f)
-	       ((null? attr) #f)
-	       ((pair? (car attr)) `(@ . ,attr))
-	       (else attr)))
-	(tail (let loop ((items rest))
-		(cond
-		 ((null? (cdr items)) (car items))
-		 ((not (car items)) (loop (cdr items)))
-		 ((null? (car items)) (loop (cdr items)))
-		 (else (cons (car items) (loop (cdr items))))))))
-    (if attr (cons* tag attr tail) (cons tag tail))))
+  (let ((tail (let loop ((tail rest))
+                (cond
+                 ((null? (cdr tail)) (car tail))
+                 ((not (car tail)) (loop (cdr tail)))
+                 ((null? (car tail)) (loop (cdr tail)))
+                 (else (cons (car tail) (loop (cdr tail))))))))
+    (sx-cons tag attr tail)))
 (define (sx-list tag attr . rest)
-  (let ((attr (cond
-	       ((not attr) #f)
-	       ((null? attr) #f)
-	       ((pair? (car attr)) `(@ . ,attr))
-	       (else attr)))
-	(tail (let loop ((items rest))
-		(cond
-		 ((null? items) '())
-		 ((not (car items)) (loop (cdr items)))
-		 ((null? (car items)) (loop (cdr items)))
-		 (else (cons (car items) (loop (cdr items))))))))
-    (if attr (cons* tag attr tail) (cons tag tail))))
+  (let ((tail (let loop ((tail rest))
+                (cond
+                 ((null? tail) '())
+                 ((not (car tail)) (loop (cdr tail)))
+                 ((null? (car tail)) (loop (cdr tail)))
+                 (else (cons (car tail) (loop (cdr tail))))))))
+    (sx-cons tag attr tail)))
+(define sx-join sx-cons)
+(define sx-join* sx-list)
 
 ;; @deffn {Procedure} sx-split sexp => tag attr tail
 ;; @deffnx {Procedure} sx-split* sexp => tag attr exp ...
@@ -284,39 +293,15 @@
 ;; @end deffn
 (define (sx-split sexp)
   (let ((tag (sx-tag sexp))
-	(attr (sx-attr sexp))
-	(tail (sx-tail sexp)))
+        (attr (sx-attr sexp))
+        (tail (sx-tail sexp)))
     (values tag attr tail)))
 (define (sx-split* sexp)
   (let ((tag (sx-tag sexp))
-	(attr (sx-attr sexp))
-	(tail (sx-tail sexp)))
+        (attr (sx-attr sexp))
+        (tail (sx-tail sexp)))
     (apply values tag attr tail)))
 
-;; @deffn {Procedure} sx-join tag attr tail => sexp
-;; @deffnx {Procedure} sx-join* tag attr exp ... => sexp
-;; Build an SXML element by its parts.  If @var{ATTR} is @code{'()} skip;
-;; @code{sx-join*} will remove any exp that are @code{#f} or @code{'()}.
-;; @end deffn
-(define (sx-join tag attr tail)
-  (if (and attr (pair? attr))
-      (if (pair? (car attr))
-	  (cons* tag `(@ . ,attr) tail)
-	  (cons* tag `(@ ,attr) tail))
-      (cons tag tail)))
-(define (sx-join* tag attr . tail)
-  (let ((tail (let loop ((tail tail))
-		(cond
-		 ((null? tail) '())
-		 ((not tail) (loop (cdr tail)))
-		 ((null? (car tail)) (loop (cdr tail)))
-		 (else (cons (car tail) (loop (cdr tail))))))))
-    (if (and attr (pair? attr))
-	(if (pair? (car attr))
-	    (cons* tag `(@ . ,attr) tail)
-	    (cons* tag `(@ ,attr) tail))
-	(cons tag tail))))
-
 ;; @deffn {Procedure} sx-unitize list-tag form seed
 ;; Given a declaration of form @code{(tag ... (elt-list ...) ...)}
 ;; fold into the seed broken list of
@@ -326,13 +311,13 @@
 (define (sx-unitize list-tag form seed)
   (let loop ((head '()) (elts '()) (tail '()) (form form))
     (if (null? elts)
-	(if (and (pair? (car form)) (eq? list-tag (sx-tag (car form))))
-	    (loop head (cdar form) (cdr form) '())
-	    (loop (cons (car form) head) elts tail (cdr form)))
-	(let loop2 ((elts elts))
-	  (if (null? elts) seed
-	      (cons (append-reverse (cons (car elts) head) tail)
-		    (loop2 (cdr elts))))))))
+        (if (and (pair? (car form)) (eq? list-tag (sx-tag (car form))))
+            (loop head (cdar form) (cdr form) '())
+            (loop (cons (car form) head) elts tail (cdr form)))
+        (let loop2 ((elts elts))
+          (if (null? elts) seed
+              (cons (append-reverse (cons (car elts) head) tail)
+                    (loop2 (cdr elts))))))))
 
 ;; ============================================================================
 
@@ -345,7 +330,7 @@
 ;; sx-haz-attr? val
 ;;(define (sx-haz-attr? sx)
 ;;  (and (pair? (cdr sx)) (pair? (cadr sx)) (eqv? '@ (caadr sx)) #t))
-	
+        
 ;; Given that a tag must be ... we define the syntax of SXML as follows:
 ;; SXML is a text format for XML using S-expressions, sexp's whose first
 ;; element is a symbol for a legal XML tag.  The syntax is:
@@ -371,8 +356,8 @@
 ;;   (foo (@ . ,<name>) (bar ,abc) ...)
 ;;   (foo (bar ,abc) ...)
 ;;   (foo ... , *)
-;;   (* ...)			any sexp
-;;   *				any node (i.e., sexp or text)
+;;   (* ...)                    any sexp
+;;   *                          any node (i.e., sexp or text)
 ;; or use `*any*'
 ;; need don't care: (foo a b c . ?) (foo a b ? c)
 ;; if expect text, must use , or ?
@@ -395,7 +380,7 @@
   (syntax-rules ()
     ((_ v (pat exp ...) c1 ...)
      (let ((kf (lambda () (sx-match-1 v c1 ...))))
-       (sxm-sexp v pat (begin exp ...) (kf))))
+       (sxm-sexp v pat (begin (if #f #f) exp ...) (kf))))
     ((_ v) (error "sx-match: nothing matches"))))
 
 (define-syntax sx-match-tail
@@ -407,7 +392,7 @@
   (syntax-rules ()
     ((_ v (pat exp ...) c1 ...)
      (let ((kf (lambda () (sx-match-1-tail v c1 ...))))
-       (sxm-tail v pat (begin exp ...) (kf))))
+       (sxm-tail v pat (begin (if #f #f) exp ...) (kf))))
     ((_ v) (error "sx-match-tail: nothing matches"))))
 
 ;; sxm-sexp val pat kt kf
@@ -416,21 +401,35 @@
   (syntax-rules (@ unquote)
     ;; accept anything
     ((_ v (unquote w) kt kf) (let ((w v)) kt))
-    ;; capture attributes
-    ((_ v (tag (@ . (unquote al)) . nl) kt kf)
+    ;; capture attributes by name
+    ((_ v (tag (@ (ky vl) p1 ...) . nl) kt kf)
      (sxm-tag (car v) tag
-	      (if (sx-has-attr? v)
-		  (let ((al (cdadr v))) (sxm-tail (cddr v) nl kt kf))
-		  (let ((al '())) (sxm-tail (cdr v) nl kt kf)))
-	      kf))
+              (let ((va (sx-attr v)) (vt (sx-tail v)))
+                (sxm-attr-tail va vt ((ky vl) p1 ...) nl kt kf))
+              kf))
+    ;; capture attributes as dict
+    ((_ v (tag (@ . (unquote va)) . nl) kt kf)
+     (sxm-tag (car v) tag
+              (let ((va (sx-attr v)) (vt (sx-tail v)))
+                (sxm-tail vt nl kt kf))
+              kf))
     ;; ignore attributes; (cadr v) may be an attr node. If so, ignore it.
     ((_ v (tag . nl) kt kf)
      (sxm-tag (car v) tag
-	      (if (sx-has-attr? v)
-		  (sxm-tail (cddr v) nl kt kf)
-		  (sxm-tail (cdr v) nl kt kf))
-	      kf))))
+              (let ((vt (sx-tail v)))
+                (sxm-tail vt nl kt kf))
+              kf))
+    ))
  
+;; sxml-attr-tail va vt (@ (k v) ...) nl kt kf
+(define-syntax sxm-attr-tail
+  (syntax-rules (unquote)
+    ((_ va vt () nl kt kf)
+     (sxm-tail vt nl kt kf))
+    ((_ va vt ((key (unquote val)) p1 ...) tl kt kf)
+     (let ((val (sx-attr-ref va 'key)))
+       (sxm-attr-tail va vt (p1 ...) tl kt kf)))))
+
 ;; sxm-tag val pat kt kf
 ;; match tag: foo|#(foo bar baz)|,any
 (define-syntax sxm-tag
@@ -455,9 +454,9 @@
     ((_ v (unquote w) kt kf) (let ((w v)) kt))
     ((_ v (hp . tp) kt kf)
      (if (pair? v)
-	 (let ((hv (car v)) (tv (cdr v)))
-	   (sxm-node hv hp (sxm-tail tv tp kt kf) kf))
-	 kf))
+         (let ((hv (car v)) (tv (cdr v)))
+           (sxm-node hv hp (sxm-tail tv tp kt kf) kf))
+         kf))
     ((_ v p kt kf) kf)))
 
 ;; [ht][vp] = [head,tail][value,pattern]
